-- <pre> Standby Current Measurement Firmware, Toplevel Unit

-- Version 1, 23-JAN-19. Based on P3036A05, all functionality other than power control
-- unit removed. All I/O set to inputs except for two. The RCK input is a clock required-- to move the power control unit through a few initial states on its way to standby.
-- The KEEPER output that we use to direct the STDBY signal to the outside world. If 
-- we don't route STDBY to a pin, the compiler eliminates the power control unit and 
-- we do not get the standby state.

library ieee;  
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_unsigned.all;

entity main is 
	port (
		INPUTS : in std_logic_vector(10 downto 0);
		RCK : in std_logic; -- Reference clock to move power controler to standby.
		KEEPER : out std_logic -- Keeper output to avoid compiler elimination of logic.
	);
end;

architecture behavior of main is
	
-- Attributes to guide the compiler.
	attribute syn_keep : boolean;
	attribute nomerge : string;

-- Power Controller
	signal USERSTDBY, CLRFLAG, STDBY : std_logic;
	attribute syn_keep of STDBY : signal is true;
	attribute nomerge of STDBY : signal is "";
-- Functions and Procedures	
	function to_std_logic (v: boolean) return std_ulogic is
	begin if v then return('1'); else return('0'); end if; end function;

begin

-- We turn off the logic chip bandgap references and other unecessary
-- circuits with the power controller unit. Within a few milliseconds
-- of power-up, the chip is fully operational and ready to enter its
-- standby mode, which is the mode we run it in all the time. The PCU
-- entity has two outputs: STDBY and SFLAG. We don't connect SFLAG,
-- but we do connect STDBY to a keeper output pin on the chip. We 
-- find that the compiler eliminates the PCU unless one of its outputs
-- is connected to a pin or a state machine.
	Power_Controller: entity PCU port map (
		CLRFLAG => CLRFLAG,
		USERSTDBY => USERSTDBY, 
		STDBY => STDBY,
		SFLAG => open);
	PowerUp: process is
		constant PowerUpEnd : integer := 3;
		variable counter : integer range 0 to PowerUpEnd := 0;
	begin
		wait until (RCK = '1');
		if (counter = PowerUpEnd) then 
			counter := counter;
			CLRFLAG <= '0';
			USERSTDBY <= '1';
		else
			counter := counter + 1;
			USERSTDBY <= '0';
			if (counter = 1) then
				CLRFLAG <= '1';
			else 
				CLRFLAG <= '0';
			end if;
		end if;
		
	end process;
	
-- The KEEPER output we use to force the compiler to retain certain signals it
-- would otherwise eliminate along with necessary logic. If STDBY is eliminated,
-- the entire power control unit is eliminated also, which stops the device
-- going into its low-power state.
	KEEPER <= STDBY or to_std_logic(INPUTS = 0);

end behavior;